@@ -27,16 +27,11 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <libpq-fe.h>
+#include <stdatomic.h>
 
-#include "URL.h"
-#include "ResultSet.h"
+#include "PostgresqlAdapter.h"
 #include "StringBuffer.h"
-#include "PreparedStatement.h"
-#include "PostgresqlResultSet.h"
-#include "PostgresqlPreparedStatement.h"
 #include "ConnectionDelegate.h"
-#include "PostgresqlConnection.h"
 
 
 /**
@@ -49,35 +44,15 @@
 /* ----------------------------------------------------------- Definitions */
 
 
-const struct Cop_T postgresqlcops = {
-        .name 		 	= "postgresql",
-        .new 		 	= PostgresqlConnection_new,
-        .free 		 	= PostgresqlConnection_free,
-        .setQueryTimeout 	= PostgresqlConnection_setQueryTimeout,
-        .setMaxRows 	 	= PostgresqlConnection_setMaxRows,
-        .ping		 	= PostgresqlConnection_ping,
-        .beginTransaction	= PostgresqlConnection_beginTransaction,
-        .commit			= PostgresqlConnection_commit,
-        .rollback		= PostgresqlConnection_rollback,
-        .lastRowId		= PostgresqlConnection_lastRowId,
-        .rowsChanged		= PostgresqlConnection_rowsChanged,
-        .execute		= PostgresqlConnection_execute,
-        .executeQuery		= PostgresqlConnection_executeQuery,
-        .prepareStatement	= PostgresqlConnection_prepareStatement,
-        .getLastError		= PostgresqlConnection_getLastError
-};
-
 #define T ConnectionDelegate_T
 struct T {
-        URL_T url;
 	PGconn *db;
-	PGresult *res;
-	int maxRows;
-	int timeout;
-	ExecStatusType lastError;
+        PGresult *res;
         StringBuffer_T sb;
+        Connection_T delegator;
+	ExecStatusType lastError;
 };
-static uint32_t statementid = 0;
+static _Atomic(uint32_t) kStatementID = 0;
 extern const struct Rop_T postgresqlrops;
 extern const struct Pop_T postgresqlpops;
 
@@ -85,53 +60,54 @@ extern const struct Pop_T postgresqlpops;
 /* ------------------------------------------------------- Private methods */
 
 
-static int _doConnect(T C, char **error) {
+static bool _doConnect(T C, char **error) {
 #define ERROR(e) do {*error = Str_dup(e); goto error;} while (0)
+        URL_T url = Connection_getURL(C->delegator);
         /* User */
-        if (URL_getUser(C->url))
-                StringBuffer_append(C->sb, "user='%s' ", URL_getUser(C->url));
-        else if (URL_getParameter(C->url, "user"))
-                StringBuffer_append(C->sb, "user='%s' ", URL_getParameter(C->url, "user"));
+        if (URL_getUser(url))
+                StringBuffer_append(C->sb, "user='%s' ", URL_getUser(url));
+        else if (URL_getParameter(url, "user"))
+                StringBuffer_append(C->sb, "user='%s' ", URL_getParameter(url, "user"));
         else
                 ERROR("no username specified in URL");
         /* Password */
-        if (URL_getPassword(C->url))
-                StringBuffer_append(C->sb, "password='%s' ", URL_getPassword(C->url));
-        else if (URL_getParameter(C->url, "password"))
-                StringBuffer_append(C->sb, "password='%s' ", URL_getParameter(C->url, "password"));
-        else
+        if (URL_getPassword(url))
+                StringBuffer_append(C->sb, "password='%s' ", URL_getPassword(url));
+        else if (URL_getParameter(url, "password"))
+                StringBuffer_append(C->sb, "password='%s' ", URL_getParameter(url, "password"));
+        else if (! URL_getParameter(url, "unix-socket"))
                 ERROR("no password specified in URL");
         /* Host */
-        if (URL_getParameter(C->url, "unix-socket")) {
-                if (URL_getParameter(C->url, "unix-socket")[0] != '/')
+        if (URL_getParameter(url, "unix-socket")) {
+                if (URL_getParameter(url, "unix-socket")[0] != '/')
                         ERROR("invalid unix-socket directory");
-                StringBuffer_append(C->sb, "host='%s' ", URL_getParameter(C->url, "unix-socket"));
-        } else if (URL_getHost(C->url)) {
-                StringBuffer_append(C->sb, "host='%s' ", URL_getHost(C->url));
+                StringBuffer_append(C->sb, "host='%s' ", URL_getParameter(url, "unix-socket"));
+        } else if (URL_getHost(url)) {
+                StringBuffer_append(C->sb, "host='%s' ", URL_getHost(url));
                 /* Port */
-                if (URL_getPort(C->url) > 0)
-                        StringBuffer_append(C->sb, "port=%d ", URL_getPort(C->url));
+                if (URL_getPort(url) > 0)
+                        StringBuffer_append(C->sb, "port=%d ", URL_getPort(url));
                 else
                         ERROR("no port specified in URL");
         } else
                 ERROR("no host specified in URL");
         /* Database name */
-        if (URL_getPath(C->url))
-                StringBuffer_append(C->sb, "dbname='%s' ", URL_getPath(C->url) + 1);
+        if (URL_getPath(url))
+                StringBuffer_append(C->sb, "dbname='%s' ", URL_getPath(url) + 1);
         else
                 ERROR("no database specified in URL");
         /* Options */
-        StringBuffer_append(C->sb, "sslmode='%s' ", IS(URL_getParameter(C->url, "use-ssl"), "true") ? "require" : "disable");
-        if (URL_getParameter(C->url, "connect-timeout")) {
+        StringBuffer_append(C->sb, "sslmode='%s' ", IS(URL_getParameter(url, "use-ssl"), "true") ? "require" : "disable");
+        if (URL_getParameter(url, "connect-timeout")) {
                 TRY
-                        StringBuffer_append(C->sb, "connect_timeout=%d ", Str_parseInt(URL_getParameter(C->url, "connect-timeout")));
+                        StringBuffer_append(C->sb, "connect_timeout=%d ", Str_parseInt(URL_getParameter(url, "connect-timeout")));
                 ELSE
                         ERROR("invalid connect timeout value");
                 END_TRY;
         } else
-                StringBuffer_append(C->sb, "connect_timeout=%d ", SQL_DEFAULT_TCP_TIMEOUT);
-        if (URL_getParameter(C->url, "application-name"))
-                StringBuffer_append(C->sb, "application_name='%s' ", URL_getParameter(C->url, "application-name"));
+                StringBuffer_append(C->sb, "connect_timeout=%d ", SQL_DEFAULT_TIMEOUT/MSEC_PER_SEC);
+        if (URL_getParameter(url, "application-name"))
+                StringBuffer_append(C->sb, "application_name='%s' ", URL_getParameter(url, "application-name"));
         /* Connect */
         C->db = PQconnectdb(StringBuffer_toString(C->sb));
         if (PQstatus(C->db) == CONNECTION_OK)
@@ -142,61 +118,47 @@ error:
 }
 
 
-/* ----------------------------------------------------- Protected methods */
+/* -------------------------------------------------------- Delegate Methods */
 
 
-#ifdef PACKAGE_PROTECTED
-#pragma GCC visibility push(hidden)
-#endif
-
-T PostgresqlConnection_new(URL_T url, char **error) {
-	T C;
-	assert(url);
-        assert(error);
-        NEW(C);
-        C->url = url;
-        C->sb = StringBuffer_create(STRLEN);
-        C->timeout = SQL_DEFAULT_TIMEOUT;
-        if (! _doConnect(C, error))
-                PostgresqlConnection_free(&C);
-	return C;
-}
-
-
-void PostgresqlConnection_free(T *C) {
-	assert(C && *C);
+static void _free(T *C) {
+        assert(C && *C);
         if ((*C)->res)
                 PQclear((*C)->res);
         if ((*C)->db)
                 PQfinish((*C)->db);
-        StringBuffer_free(&(*C)->sb);
-	FREE(*C);
+        StringBuffer_free(&((*C)->sb));
+        FREE(*C);
 }
 
 
-void PostgresqlConnection_setQueryTimeout(T C, int ms) {
-	assert(C);
-        C->timeout = ms;
-        StringBuffer_set(C->sb, "SET statement_timeout TO %d;", C->timeout);
-        PGresult *res = PQexec(C->db, StringBuffer_toString(C->sb));
-        PQclear(res);
+static T _new(Connection_T delegator, char **error) {
+	T C;
+	assert(delegator);
+        assert(error);
+        NEW(C);
+        C->delegator = delegator;
+        C->sb = StringBuffer_create(STRLEN);
+        if (! _doConnect(C, error))
+                _free(&C);
+	return C;
 }
 
 
-void PostgresqlConnection_setMaxRows(T C, int max) {
-	assert(C);
-        C->maxRows = max;
-}
-
-
-int PostgresqlConnection_ping(T C) {
+static bool _ping(T C) {
         assert(C);
         return (PQstatus(C->db) == CONNECTION_OK);
 }
 
 
+static void _setQueryTimeout(T C, int ms) {
+        assert(C);
+        StringBuffer_set(C->sb, "SET statement_timeout TO %d;", ms);
+        PQclear(PQexec(C->db, StringBuffer_toString(C->sb)));
+}
 
-int PostgresqlConnection_beginTransaction(T C) {
+
+static bool _beginTransaction(T C) {
 	assert(C);
         PGresult *res = PQexec(C->db, "BEGIN TRANSACTION;");
         C->lastError = PQresultStatus(res);
@@ -205,7 +167,7 @@ int PostgresqlConnection_beginTransaction(T C) {
 }
 
 
-int PostgresqlConnection_commit(T C) {
+static bool _commit(T C) {
 	assert(C);
         PGresult *res = PQexec(C->db, "COMMIT TRANSACTION;");
         C->lastError = PQresultStatus(res);
@@ -214,7 +176,7 @@ int PostgresqlConnection_commit(T C) {
 }
 
 
-int PostgresqlConnection_rollback(T C) {
+static bool _rollback(T C) {
 	assert(C);
         PGresult *res = PQexec(C->db, "ROLLBACK TRANSACTION;");
         C->lastError = PQresultStatus(res);
@@ -223,23 +185,23 @@ int PostgresqlConnection_rollback(T C) {
 }
 
 
-long long PostgresqlConnection_lastRowId(T C) {
+static long long _lastRowId(T C) {
         assert(C);
         return (long long)PQoidValue(C->res);
 }
 
 
-long long PostgresqlConnection_rowsChanged(T C) {
+static long long _rowsChanged(T C) {
         assert(C);
         char *changes = PQcmdTuples(C->res);
         return changes ? Str_parseLLong(changes) : 0;
 }
 
 
-int PostgresqlConnection_execute(T C, const char *sql, va_list ap) {
-        va_list ap_copy;
+static bool _execute(T C, const char *sql, va_list ap) {
 	assert(C);
         PQclear(C->res);
+        va_list ap_copy;
         va_copy(ap_copy, ap);
         StringBuffer_vset(C->sb, sql, ap_copy);
         va_end(ap_copy);
@@ -249,49 +211,63 @@ int PostgresqlConnection_execute(T C, const char *sql, va_list ap) {
 }
 
 
-ResultSet_T PostgresqlConnection_executeQuery(T C, const char *sql, va_list ap) {
-        va_list ap_copy;
+static ResultSet_T _executeQuery(T C, const char *sql, va_list ap) {
 	assert(C);
         PQclear(C->res);
+        va_list ap_copy;
         va_copy(ap_copy, ap);
         StringBuffer_vset(C->sb, sql, ap_copy);
         va_end(ap_copy);
         C->res = PQexec(C->db, StringBuffer_toString(C->sb));
         C->lastError = PQresultStatus(C->res);
         if (C->lastError == PGRES_TUPLES_OK)
-                return ResultSet_new(PostgresqlResultSet_new(C->res, C->maxRows), (Rop_T)&postgresqlrops);
+                return ResultSet_new(PostgresqlResultSet_new(C->delegator, C->res), (Rop_T)&postgresqlrops);
         return NULL;
 }
 
 
-PreparedStatement_T PostgresqlConnection_prepareStatement(T C, const char *sql, va_list ap) {
-        char *name;
-        int paramCount = 0;
-        va_list ap_copy;
+static PreparedStatement_T _prepareStatement(T C, const char *sql, va_list ap) {
         assert(C);
         assert(sql);
         PQclear(C->res);
+        va_list ap_copy;
         va_copy(ap_copy, ap);
         StringBuffer_vset(C->sb, sql, ap_copy);
         va_end(ap_copy);
-        paramCount = StringBuffer_prepare4postgres(C->sb);
-        uint32_t t = ++statementid; // increment is atomic
-        name = Str_cat("%d", t);
+        int paramCount = StringBuffer_prepare4postgres(C->sb);
+        uint32_t t = kStatementID++; // increment is atomic
+        char *name = Str_cat("__libzdb-%d", t);
         C->res = PQprepare(C->db, name, StringBuffer_toString(C->sb), 0, NULL);
         C->lastError = C->res ? PQresultStatus(C->res) : PGRES_FATAL_ERROR;
         if (C->lastError == PGRES_EMPTY_QUERY || C->lastError == PGRES_COMMAND_OK || C->lastError == PGRES_TUPLES_OK)
-		return PreparedStatement_new(PostgresqlPreparedStatement_new(C->db, C->maxRows, name, paramCount), (Pop_T)&postgresqlpops, paramCount);
+		return PreparedStatement_new(PostgresqlPreparedStatement_new(C->delegator, C->db, name, paramCount), (Pop_T)&postgresqlpops);
         return NULL;
 }
 
 
-const char *PostgresqlConnection_getLastError(T C) {
+static const char *_getLastError(T C) {
 	assert(C);
         return C->res ? PQresultErrorMessage(C->res) : "unknown error";
 }
 
 
-#ifdef PACKAGE_PROTECTED
-#pragma GCC visibility pop
-#endif
+/* ------------------------------------------------------------------------- */
+
+
+const struct Cop_T postgresqlcops = {
+        .name             = "postgresql",
+        .new              = _new,
+        .free             = _free,
+        .ping             = _ping,
+        .setQueryTimeout  = _setQueryTimeout,
+        .beginTransaction = _beginTransaction,
+        .commit           = _commit,
+        .rollback         = _rollback,
+        .lastRowId        = _lastRowId,
+        .rowsChanged      = _rowsChanged,
+        .execute          = _execute,
+        .executeQuery     = _executeQuery,
+        .prepareStatement = _prepareStatement,
+        .getLastError     = _getLastError
+};
 
